/*
 *  macdump/ mac address dumper
 *  Copyright 2002,2003 Junichi Uekawa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
  MAC address dumping routine. 
  This application requires ROOT privilage to access all packets that 
  the host receives.
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <netpacket/packet.h>
#include <getopt.h>
#include <string.h>

#ifndef ETH_P_ALL
/* from linux kernel source */
#define ETH_P_ALL 0x0003
#endif

/* from linux kernel source */
#define ETH_P_IP 0x0800

#include "config.h"
#include "update-cluster-i18n.h"

char * ipprefix="192.168.0.";
char * prefix="node";
int ipstart=2;
int hoststart=2;
char * domainsuffix=".cluster";
char * interface=NULL;
char * clusterxml= PKGSYSCONFDIR "/cluster.xml";

static void 
showhelp ()
{
  printf (_("update-cluster-maccollect [--options] [item] ... \n"
	    "  collects mac address using tcpdump, and creates necessary hostnames \n"
	    "  and IP address\n"
	    "  for the MAC address.\n"
	    "\n"));
}

/**
   Parse the options and work.
   @return 0 success
   @return 1 failure to parse
 */
static int
parse_options(int ac, char ** av )
{
  int index_point;  
  int c;
  
  static struct option long_options[]=
  {
    {"prefix", required_argument, 0, 'p'},
    {"ipprefix", required_argument, 0, 'i'},
    {"ipstart", required_argument, 0, 'I'},
    {"hoststart", required_argument, 0, 'H'},
    {"domainsuffix", required_argument, 0, 'd'},
    {"interface", required_argument, 0, 'e'},
    {"clusterxml", required_argument, 0, 'c'},
    {"help", no_argument, 0, 'h'},
    {0,0,0,0}
  };

  while((c = getopt_long (ac, av, 
			  "p:i:I:H:d:e:h", 
			  long_options, &index_point)) != -1)
    {
      switch (c)
	{
	case 'p': 			/* --prefix */
	  prefix=strdup(optarg);
	  break;
	case 'i': 			/* --ipprefix */
	  ipprefix=strdup(optarg);
	  break;
	case 'I': 			/* --ipstart */
	  ipstart=atoi(optarg);
	  break;
	case 'H': 			/* --hoststart */
	  hoststart=atoi(optarg);
	  break;
	case 'd': 			/* --domainsuffix */
	  domainsuffix=strdup(optarg);
	  break;
	case 'e': 			/* --interface */
	  interface=strdup(optarg);
	  break;
	case 'c': 			/* --clusterxml */
	  clusterxml=strdup(optarg);
	  break;
	case 'h': 			/* --help */
	default:
	  showhelp();
	  return 1;
	}
    }
  return 0;
}

int 
main(int ac, char ** av)
{
  //int s = socket (AF_INET, SOCK_PACKET, htons(ETH_P_IP));
  int s = socket (AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
  struct sockaddr_ll sll;
  unsigned char f[BUFSIZ+2];
  int ifindex;
  int count;
  char * mac = NULL;
  char * execstring = NULL;

  setlocale(LC_ALL, "");
  bindtextdomain(PACKAGE_NAME, LOCALEDIR);
  textdomain(PACKAGE_NAME);

  if (parse_options(ac,av))
    exit (1);

  
  memset(&sll, 0, sizeof(sll));
  sll.sll_family          = AF_PACKET;
  if (interface) 
    {
      ifindex = if_nametoindex(interface);
      sll.sll_ifindex         = ifindex;
      printf(_("Interface set to %s\n"), interface);
    }
  sll.sll_protocol        = htons(ETH_P_ALL);
  if (bind(s, (struct sockaddr *) &sll, sizeof(sll)) == -1) 
    {
      perror("update-cluster-maccollect");
      exit(1);
    }

  while ((count=read(s,f,BUFSIZ)) >= 0)
    {
      /* first 6 bytes contain the MAC address of the packet sender. */
      
      if (count > 6)
	{
	  if (asprintf(&mac,"%x:%x:%x:%x:%x:%x",f[0],f[1],f[2],f[3],f[4],f[5]) < 0)
	    {
	      /* error */
	      fprintf(stderr,_("Out of memory in asprintf\n"));
	      exit (1);
	    }
	  else
	    {
	      if (asprintf(&execstring, "update-cluster-remove --not --failfound --mac %s > /dev/null < %s", mac, clusterxml) < 0)
		{
		  /* error */
		  fprintf(stderr,_("Out of memory in asprintf\n"));
		  exit (1);
		}
	      else
		{
		  if (system (execstring))
		    {
		      printf (_("Mac address %s already exists\n"), mac);
		    }
		  else
		    {
		      printf (_("Add MAC address %s\n"), mac);
		      fflush (stdout);

		      asprintf(&execstring,
			       "update-cluster-add --ip %s%i --hostname %s%i%s --mac %s --clusterxml %s",
			       ipprefix, ipstart, prefix, hoststart, domainsuffix, 
			       mac,
			       clusterxml);
		      if (system(execstring))
			{
			  /* error */
			  fprintf(stderr,_("Error in system call to [%s]\n"), execstring);
			  exit (1);
			}
		      ipstart ++ ;
		      hoststart ++;
		    }
		}	      
	    }
	}
    }
  return 0;
}
